package model

import "encoding/json"

// 卡关联信息
type Iot_sim_map struct {
	ModelUUID
	Iccid     string `gorm:"not null;type:varchar(255)" json:"iccid"`     // ICCID
	MapSource string `gorm:"not null;type:varchar(255)" json:"mapSource"` // 映射来源
	Source    string `gorm:"not null;type:varchar(255)" json:"source"`    // 来源
	TariffId  string `gorm:"not null;type:varchar(255)" json:"tariffId"`  // 资费
	// 创建人
	CreateUserId uint `json:"createUserId"`
	// 更新人
	UpdateUserId uint `json:"updateUserId"`
	// 删除人
	DeleteUserId uint `json:"deleteUserId"`
}

// SIM卡
type Sim_card struct {
	ModelUUID
	Iccid               string `gorm:"not null;type:varchar(255)" json:"iccid"`      // ICCID
	Source              string `gorm:"not null;type:varchar(255)" json:"source"`     // 来源
	CurrentImsi         string `gorm:"type:varchar(255)" json:"currentImsi"`         // 当前 IMSI
	BindImsi            string `gorm:"type:varchar(255)" json:"bindImsi"`            // 当前 SIM 卡绑定的 IMSI,多个以逗号隔开
	MoneyBalances       string `gorm:"type:varchar(255)" json:"moneyBalances"`       // SIM 卡货币余额
	CreateTime          string `gorm:"type:varchar(255)" json:"createTime"`          // SIM 卡生成日期
	PayType             string `gorm:"type:varchar(255)" json:"payType"`             // 支付方式 0:Prepay  1:Postpay
	IccidStatus         string `gorm:"type:varchar(255)" json:"iccidStatus"`         // SIM 卡状态
	DataUsageTotal      string `gorm:"type:varchar(255)" json:"dataUsageTotal"`      // 已使用总流量(单位：MB)
	VoiceMtTotal        string `gorm:"type:varchar(255)" json:"voiceMtTotal"`        // 语音呼入分钟数
	VoiceMoTotal        string `gorm:"type:varchar(255)" json:"voiceMoTotal"`        // 语音呼出分钟数
	VoiceTotal          string `gorm:"type:varchar(255)" json:"voiceTotal"`          // 总的语音分钟数
	SmsTotal            string `gorm:"type:varchar(255)" json:"smsTotal"`            // 发短信数
	ValidMonth          string `gorm:"type:varchar(255)" json:"validMonth"`          // SIM 卡有效期(单位:月)
	CloseTime           string `gorm:"type:varchar(255)" json:"closeTime"`           // SIM 卡关闭日期
	ActiveTime          string `gorm:"type:varchar(255)" json:"activeTime"`          // SIM 卡激活日期
	CurrentImsiProvider string `gorm:"type:varchar(255)" json:"currentImsiProvider"` // 当前 IMSI 所属供应商名称
	DataSpeed           string `gorm:"type:varchar(255)" json:"dataSpeed"`           // 默认限速

	UserId       uint   `json:"userId"`                            // 用户ID
	TariffId     string `gorm:"type:varchar(255)" json:"tariffId"` // 资费ID
	PoolId       string `gorm:"type:varchar(255)" json:"poolId"`   // 流量池Id
	CreateUserId uint   `json:"createUserId"`                      // 创建人
	UpdateUserId uint   `json:"updateUserId"`                      // 更新人
}

// SIM 绑定的套餐
type Sim_package struct {
	ModelUUID
	Iccid        string `gorm:"type:varchar(255);column:iccid;" json:"ICCID"` // ICCID
	TId          string `gorm:"type:varchar(255)" json:"tId"`                 // 订单ID
	ProductId    string `gorm:"type:varchar(255)" json:"productId"`           // 流量包ID
	OTWProductId int    `json:"otwProductId"`                                 // OTW流量包ID
	ProductName  string `gorm:"type:varchar(255)" json:"productName"`         // 流量包名称
	Status       string `gorm:"type:varchar(255)" json:"status"`              // 状态  套餐状态 0:Inactive  1:Activated  2:Close 3:Expired
	CreateTime   string `json:"createTime"`                                   // 套餐生成日期
	ActiveTime   string `json:"activeTime"`                                   // 套餐激活时间
	ExpiryTime   string `json:"expiryTime"`                                   // 套餐过期时间
	DataTotal    int    `json:"dataTotal"`                                    // 套餐可用流量：-1表示无限流量(单位: MB)
	DataUsage    int    `json:"dataUsage"`                                    // 套餐已使用流量(单位：MB)
	DataToday    int    `json:"dataToday"`                                    // 套餐今日使用流量(单位：MB)
	ValidDays    int    `json:"validDays"`                                    // 套餐有效天数
	Present      int    `json:"present"`                                      // 是否赠送套餐 0-否  1-是
}

// SIM套餐
type Metadata_package struct {
	ModelUUID
	Source       string `gorm:"not null;type:varchar(255)" json:"source"`      // 来源
	ProductId    string `gorm:"not null;type:varchar(255)" json:"productId"`   // 流量包ID
	ProductName  string `gorm:"not null;type:varchar(255)" json:"productName"` // 流量包名称
	DataZoneId   string `gorm:"type:varchar(255)" json:"dataZoneId"`           // 地区ID
	DataZoneName string `gorm:"type:varchar(255)" json:"dataZoneName"`         // 地区名称

	ValidDays        int             `json:"validDays"`                  // 套餐有效天数
	DataTotal        int             `json:"dataTotal"`                  // 套餐可用流量：-1 表示无限流量(单位：MB)
	VoiceMt          int             `json:"voiceMt"`                    // 语音呼入流量
	VoiceMo          int             `json:"voiceMo"`                    // 语音呼出流量
	SmsTotal         int             `json:"smsTotal"`                   // 短信总条数
	DataSpeedDefault int             `json:"dataSpeedDefault"`           // 默认限速
	DataQuota        int             `json:"dataQuota"`                  // 单次请求可用流量
	ValidDayType     int             `json:"validDayType"`               // 有效期类型: 0- 24 hours 1- Natural Day
	RateGroupName    string          `json:"rateGroupName"`              // 费率组名称
	ZoneVoiceMtName  string          `json:"zoneVoiceMtName"`            // 语音呼出地区名称
	ZoneVoiceMoName  string          `json:"zoneVoiceMoName"`            // 语音呼入地区名称
	ZoneDataName     string          `json:"zoneDataName"`               // 流量使用地区名称
	ZoneSmsName      string          `json:"zoneSmsName"`                // 短信使用地区名称
	Operator         json.RawMessage `gorm:"type:json;" json:"operator"` // 运营商信息
}

// SIM 流量消耗明细
type Sim_data_usage struct {
	ModelUUID
	Iccid     string  `gorm:"not null;type:varchar(255)" json:"iccid"`     // ICCID
	TId       string  `gorm:"not null;type:varchar(255)" json:"tId"`       // 订单ID
	ProductId string  `gorm:"not null;type:varchar(255)" json:"productId"` // 流量包ID
	TariffId  string  `gorm:"not null;type:varchar(255)" json:"packageId"` // 资费Id
	PoolId    string  `gorm:"not null;type:varchar(255)" json:"poolId"`    // 流量池Id
	DataUsage int     `json:"dataUsage"`                                   // 套餐已使用流量(单位：MB)
	UserId    uint    `json:"userId"`                                      // 用户ID
	Amount    float64 `json:"amount"`                                      // 金额
}
